package acube.gram.backend.birt;


import java.io.InputStream;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.core.framework.IPlatformContext;
import org.eclipse.birt.core.framework.Platform;
import org.eclipse.birt.core.framework.PlatformServletContext;
import org.eclipse.birt.report.engine.api.EngineConfig;
import org.eclipse.birt.report.engine.api.EngineException;
import org.eclipse.birt.report.engine.api.HTMLEmitterConfig;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportEngineFactory;
import org.eclipse.birt.report.engine.api.IReportRunnable;

import acube.framework.vo.ErrorVO;
import acube.gram.backend.Backend;
import acube.gram.exception.GramErrorCodes;
import acube.gram.exception.GramException;
import acube.gram.frontend.ActionHandler;
import acube.gram.frontend.ResourceHandler;

/**
 * Implmentation de Backend pour BIRT
 * @author MAE
 *
 */
public class BirtWrapper implements Backend {

	/**
	 * le moteur de rapports BIRT
	 */
	private IReportEngine engine;
	/**
	 * cre des URL pour les actions utilisateur
	 */
	private ActionHandler actionHandler;
	/**
	 * stocke les images dans un cache et cre les URL correspondantes
	 */
	private ResourceHandler resourceHandler;
	/**
	 * le chemin dans le ServletContext
	 */
	private String path;
	//private VirtualDirectory virtualDirectory;

	
	
	

	public void setActionHandler(ActionHandler actionHandler)
			throws GramException {
		
		this.actionHandler = actionHandler;
	}

	public void setResourceHandler(ResourceHandler resourceHandler)
			throws GramException {
		
		this.resourceHandler = resourceHandler;
	}

	public void setPath(String path) {
		this.path = path;
		if (!this.path.endsWith("/"))
			this.path += "/";
	}

	public BirtExecutionContext loadReport(HttpServletRequest request, String rapport)
			throws GramException {

		try {
			// initialisation du service si besoin
			if (engine==null)
				createEngine(request.getSession().getServletContext());
			
			// TODO utiliser le VirtualDirectory au lieu ServletContext
			InputStream is = request
				.getSession()
				.getServletContext()
				.getResourceAsStream(path+rapport);
			
			if (is==null) {
				ErrorVO error = new ErrorVO();
				error.setCdError(GramErrorCodes.INVALID_REPORT);
				error.setParamLabel(rapport);
				error.setParamNota(path+rapport);
				throw new GramException(error);
			}
			
			IReportRunnable runnable = engine.openReportDesign(is);
			
			return new BirtExecutionContext(engine,request.getSession(),runnable);

		} catch (EngineException e) {
			ErrorVO vo = new ErrorVO();
			vo.setCdError(GramErrorCodes.SERVICE_ERROR);
			vo.setParamLabel(e.getMessage());
			throw new GramException(vo);
		}
	}
	
	
	

	/**
	 * Dmarre la plateforme OSGi et initialise le HTMLEmitter
	 * (gestion des images et des actions).
	 * @param servletContext
	 */
	@SuppressWarnings("unchecked")
	private synchronized void createEngine(ServletContext servletContext) {
		
		EngineConfig config = new EngineConfig();
		config.setEngineHome(""); //TODO : que devrait-on mettre ici ?

		IPlatformContext context = new PlatformServletContext(servletContext);
		config.setPlatformContext(context);
		try {
			Platform.startup(config);
		} catch (BirtException e) {
			e.printStackTrace();
		}

		HTMLEmitterConfig emitterConfig = new HTMLEmitterConfig();
		emitterConfig.setActionHandler(new GramHTMLActionHandler(actionHandler));
		emitterConfig.setImageHandler(new GramHTMLImageHandler(resourceHandler));
		config.getEmitterConfigs().put("html", emitterConfig);
		
		IReportEngineFactory factory = (IReportEngineFactory) Platform
				.createFactoryObject(IReportEngineFactory.EXTENSION_REPORT_ENGINE_FACTORY);
		engine = factory.createReportEngine(config);
	}


	/*
	public ArrayList<FileMetadata> listFiles() throws GramException {

		return virtualDirectory.listFiles("");
	}
	
	public void createFile(FileType type, String name, String contentType, byte[] definition)
			throws GramException {

		virtualDirectory.createFile(type,name,definition);
	}
	
	public FileVO readFile(FileType type, String name)
			throws GramException {
		
		return virtualDirectory.getContent(name);
	}
	
	public void updateFile(FileType type, String name, String contentType, byte[] definition)
			throws GramException {
		
		virtualDirectory.setContent(name, definition);
	}
	
	public void deleteFile(FileType type, String name)
			throws GramException {
		
		virtualDirectory.remove(name);
	}
	*/
}
