package acube.gram.backend.birt;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.birt.report.engine.api.HTMLActionHandler;
import org.eclipse.birt.report.engine.api.IAction;
import org.eclipse.birt.report.model.api.util.ParameterValidationUtil;

import acube.gram.constant.ReportFormatCodec;
import acube.gram.exception.GramException;
import acube.gram.frontend.ActionHandler;

public class GramHTMLActionHandler extends HTMLActionHandler {

	private Logger logger = Logger.getLogger(this.getClass());
	private String THIS_CLASS = "["+this.getClass().getName()+"] ";
	
	/**
	 * le hendler fourni par le front-end
	 */
	private ActionHandler handler;
	
	public GramHTMLActionHandler(ActionHandler handler) {

		this.handler = handler;
	}

	@Override
	protected String buildDrillAction(IAction action, Object context) {
		
		// rcupre le rapport et le format
		String report = action.getReportName();
		String format = action.getFormat();
		
		Map<String,String> parameters = new HashMap<String,String>();

		// paramtres du rapport
		if ( action.getParameterBindings( ) != null )
		{
			Iterator i = action.getParameterBindings().entrySet().iterator();
			while (i.hasNext()) {
				Map.Entry entry = (Map.Entry)i.next();
				String key = (String)entry.getKey();
				Object valueObj = entry.getValue();
				if (valueObj == null)
					continue;

				parameters.put(key, encode(valueObj));
			}
		}
		
		try {
			// cre l'URL
			return handler.handleDrillThrough(report, ReportFormatCodec.decode(format), parameters);
			
		} catch (GramException e) {
			logger.debug(THIS_CLASS + ": "+e.getMessage());
			return null;
		}
	}

	private String encode(Object value)
	{
		if ( value == null )
			return null;

		if ( value instanceof Float || value instanceof Double
				|| value instanceof BigDecimal )
		{
			return value.toString( );
		}
		return ParameterValidationUtil.getDisplayValue( value );
	}
}
