package acube.gram.backend.ssrs;


import java.rmi.RemoteException;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import acube.gram.backend.Backend;
import acube.gram.backend.ExecutionContext;
import acube.gram.exception.GramException;
import acube.gram.exception.GramExceptionFactory;
import acube.gram.frontend.ActionHandler;
import acube.gram.frontend.ResourceHandler;

import com.microsoft.schemas.sqlserver._2005._06._30.reporting.reportingservices.ExecutionInfo;
import com.microsoft.schemas.sqlserver._2005._06._30.reporting.reportingservices.ReportExecutionServiceSoapStub;

/**
 * implmentation de Backend pour Reporting Services
 * @author MAE
 *
 */
public class SSRSWrapper implements Backend {

	private static final String THIS_CLASS = "[" + SSRSWrapper.class.getName() + "]";
	static Logger logger = Logger.getLogger(SSRSWrapper.class);

	private SOAPStubPool pool = new SOAPStubPool();
	private String path;
	private ActionHandler actionHandler;
	private ResourceHandler resourceHandler;

	
	public void setActionHandler(ActionHandler actionHandler)
			throws GramException {

		this.actionHandler = actionHandler;
	}
	
	public void setResourceHandler(ResourceHandler resourceHandler)
			throws GramException {

		this.resourceHandler = resourceHandler;
	}
	
	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public String getUrl() {
		return pool.getUrl();
	}

	public void setUrl(String url) {
		pool.setUrl(url);
	}

	public void setUser(String login) {
		pool.setUser(login);
	}

	public void setPassword(String password) {
		pool.setPassword(password);
	}


	
	public String getAbsolutePath(String name) {
		
		if (name.startsWith(path))
			return name;
		return path+name;
	}

	/*
	public List<acube.gram.vo.FileMetadata> listFiles()
			throws GramException {

		logger.info(THIS_CLASS + "listFiles "+path);
		ReportingService2005SoapStub service = pool.getReportingService2005SoapStub();
		// list the items
		CatalogItem items[];
		try {
			items = service.listChildren(path,false);
			
		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
		
		// transform them
		ArrayList<FileMetadata> rapports = new ArrayList<FileMetadata>();
		for (int i=0; i<items.length; i++) {
			CatalogItem item = items[i];
			FileType type = null;
			if (item.getType().equals(ItemTypeEnum.Resource))
				type = FileType.RESOURCE;
			else if (item.getType().equals(ItemTypeEnum.Report))
				type = FileType.REPORT_DEFINITION;
			else if (item.getType().equals(ItemTypeEnum.DataSource))
				type = FileType.SHARED_DATASOURCE;

			if (type != null) {
				FileMetadata file = new FileMetadata();
				file.setFileType(type);
				file.setPath(item.getPath());
				file.setName(item.getDescription());
				file.setMimeType(item.getMimeType());
				rapports.add(file);
			}
		}
		return rapports;
	}
	
	public void createFile(FileType type, String name, String contentType, byte[] definition)
			throws GramException {
		
		logger.info(THIS_CLASS + "createFile " + path+name);
		
		ReportingService2005SoapStub service = pool.getReportingService2005SoapStub();
		Property properties[] = new Property[0];
		try {
			switch (type) {
			case REPORT_DEFINITION:
				// cre le rapport (ou crase celui qui existait)
				service.createReport(name, path, true, definition, properties);

				// corrige les sources de donnes
				DataSource dataSources[] = service.getItemDataSources(path);
				for (DataSource dataSource : dataSources) {
					if (dataSource.getInvalidDataSourceReference()==null)
						continue;
					dataSource.setDataSourceReference(new DataSourceReference(path+dataSource.getName()));
					dataSource.setInvalidDataSourceReference(null);
				}
				service.setItemDataSources(path, dataSources);
				break;
				
			case RESOURCE:
				service.createResource(name, path, true, definition, contentType, properties);
				break;
			}

		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
	}
	
	public FileVO readFile(FileType type, String name)
			throws GramException {

		String absolutePath = getAbsolutePath(name);
		logger.info(THIS_CLASS + "readFile " + absolutePath);
		ByteArrayHolder contentHolder = new ByteArrayHolder();
		StringHolder contentTypeHolder = new StringHolder();
		
		ReportingService2005SoapStub service = pool.getReportingService2005SoapStub();
		try {
			switch (type) {
			case REPORT_DEFINITION:
				contentHolder.value = service.getReportDefinition(absolutePath);
				contentTypeHolder.value = "text/xml";
				break;
			case RESOURCE:
				service.getResourceContents(absolutePath,contentHolder,contentTypeHolder);
				break;
			default:
				break;
			}
			
		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
		
		FileVO vo = new FileVO();
		vo.setFileName(name);
		vo.setContentType(contentTypeHolder.value);
		vo.setContent(contentHolder.value);
		return vo;
	}
	
	public void updateFile(FileType type, String name, String contentType, byte[] definition)
			throws GramException {

		String absolutePath = getAbsolutePath(name);
		logger.info(THIS_CLASS + "updateFile " + absolutePath);
		ReportingService2005SoapStub service = pool.getReportingService2005SoapStub();
		try {
			switch (type) {
			case REPORT_DEFINITION:
				service.setReportDefinition(absolutePath,definition);
				break;
			case RESOURCE:
				service.setResourceContents(absolutePath,definition,contentType);
				break;
			default:
				break;
			}
			
		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
	}
	
	public void deleteFile(FileType type, String name)
			throws GramException {

		String absolutePath = getAbsolutePath(name);
		logger.info(THIS_CLASS + "deleteFile " + absolutePath);
		ReportingService2005SoapStub service = pool.getReportingService2005SoapStub();

		try {
			service.deleteItem(absolutePath);
			
		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
	}
	*/
	public ExecutionContext loadReport(HttpServletRequest request, String name)
			throws GramException {

		String absolutePath = getAbsolutePath(name);
		logger.info(THIS_CLASS + "loadReport " + absolutePath);
		ReportExecutionServiceSoapStub service = pool.getReportExecutionServiceSoapStub();

		try {
			ExecutionInfo info = service.loadReport(absolutePath,null);
			return new SSRSExecutionContext(pool, request, info, actionHandler, resourceHandler);
			
		} catch (RemoteException e) {
			logger.error(THIS_CLASS + "RemoteException: " + e.getMessage());
			throw GramExceptionFactory.createServiceError(e);
		}
	}
}
