package acube.gram.business;

import javax.servlet.http.HttpServletRequest;

import acube.gram.backend.ExecutionContext;
import acube.gram.exception.GramException;

/**
 * A l'initialisation de GRaM, une instance de cette interface est stock&eacute;e dans le ServletContext.
 * Les actions du front-end appellent ses mthodes pour crer des contextes d'excution : pour ajouter des
 * rgles de gestion spcifiques, il suffit de crer un <i>delegate</i> sur l'interface ExecutionContext 
 * et d'encapsuler le contexte fourni par le Backend. Par exemple:
 * <ul>
 * <li>gestion des droits,</li>
 * <li>filtrage et rorganisation des paramtres</li>
 * <li>gestion du multi-application (mode "portail")</li>
 * <li>etc</li>
 * </ul>
 * Nota: dans la version complte de GRaM, avec gestion " chaud" des rapports, cette interface
 * contiendra plus de mthodes:
 * <ul>
 * <li>gestion des fichiers</li>
 * <li>gestion des droits sur les rapports</li>
 * <li>gestion de la prsentation des rapports (arborescence)</li>
 * </ul>
 * 
 * @author MAE
 * @see acube.gram.backend.Backend
 * @see acube.gram.backend.ExecutionContext
 */
public interface Delegate {

	/**
	 * Initialise un context d'ex&eacute;cution pour un rapport.
	 * @param request la requte initiale
	 * @param rapport le nom du rapport
	 * @return un nouveau contexte d'excution pour le rapport demand
	 * @throws GramException
	 */
	public ExecutionContext loadReport(HttpServletRequest request, String rapport)
		throws GramException;


	/*
	public List<FileMetadata> listFiles(HttpServletRequest request, FileType type)
		throws GramException ;
	
	public void createFile(HttpServletRequest request, FileType type, String chemin, String mimeType, byte[] definition)
		throws GramException;
	
	public FileVO readFile(HttpServletRequest request, FileType type, String racine)
		throws GramException;
	
	public void updateFile(HttpServletRequest request, FileType type, String chemin, String mimeType, byte[] definition)
		throws GramException;
	
	public void deleteFile(HttpServletRequest request, FileType type, String chemin)
		throws GramException;
	*/
	
	

	/*
	public List<RoleVO> listRoles(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public ReportVO getReportById(HttpServletRequest request, String id)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public List<ReportVO> listReports(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void createReport(HttpServletRequest request, String path, String name, List<String> roles)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void updateReport(HttpServletRequest request, String path, String name, List<String> roles)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void deleteReport(HttpServletRequest request, String path)
		throws TechnicalException, JDBCWrapperException, DAOException;
	*/
	
	
	
	/*
	public List<NodeVO> listNodes(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void createNode(HttpServletRequest request, Integer parent, String name, List<String> reports)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void updateNode(HttpServletRequest request, int id, String name, List<String> reports)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public void deleteNode(HttpServletRequest request, int id)
		throws TechnicalException, JDBCWrapperException, DAOException;
	*/
}
