package acube.gram.frontend.action;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import acube.framework.exception.AuthentificationException;
import acube.framework.exception.DAOException;
import acube.framework.exception.FunctionalException;
import acube.framework.exception.TechnicalException;
import acube.framework.vo.ErrorVO;
import acube.gram.vo.FileVO;

import com.cappuccinonet.strutscx.util.StrutsCXConstants;

/**
 * BaseAction pour renvoyer un FileVO ou forwarder a StrutsCX en cas d'erreur.
 * TODO: rintgrer au framework ACube.
 * @author MAE
 *
 */
public abstract class FileBaseAction extends Action {

	protected Logger logger = Logger.getLogger(this.getClass());
	protected final String THIS_CLASS = "["+this.getClass().getCanonicalName()+"] ";
	
	protected abstract FileVO getFileVO(HttpServletRequest request,
								 		HttpServletResponse response)
		throws IOException, DAOException, TechnicalException, FunctionalException, AuthentificationException;
	
	public ActionForward execute(ActionMapping mapping,
								 ActionForm form,
								 HttpServletRequest request,
								 HttpServletResponse response)
			throws IOException, DAOException, TechnicalException, FunctionalException {

    	ArrayList<Serializable> errors = new ArrayList<Serializable>() ;

        try {
        	FileVO vo = getFileVO(request,response);
        	
    		String contentType = vo.getContentType();
    		response.setContentType(contentType);
    		if (vo.getFileName()!=null && contentType!=null && !contentType.equalsIgnoreCase("text/html"))
    			response.setHeader("content-disposition", "attachment; filename="+vo.getFileName());

    		// On fixe la politique de cache dans l'entete HTTP
    		response.setHeader("Cache-control", "private,max-age=0");
    		
			byte content[] = vo.getContent();
			response.setContentLength(content.length);
			response.getOutputStream().write(content);
        	return null;
        	
    	} catch (TechnicalException e) {
    		ErrorVO error = e.getError();
   			logger.error(THIS_CLASS+"TechnicalException : "
   							+ ((error!=null)?error.getCdError():e.getMessage()));
    		errors.add(error);
    	} catch (FunctionalException e) {
    		ErrorVO error = e.getError();
    		logger.error(THIS_CLASS+"FunctionalException : " 
    						+ ((error!=null)?error.getCdError():e.getMessage()));
    		errors.add(error);
    	} catch (AuthentificationException e) {
    		ErrorVO error = e.getError();
    		logger.error(THIS_CLASS+"AuthentificationException : " 
    						+ ((error!=null)?error.getCdError():e.getMessage()));
    		errors.add(error);
    	}

    	// en cas d'erreur
    	response.setContentType("text/xml");
        request.setAttribute(StrutsCXConstants.ENCODING_KEY, "utf-8");
    	request.setAttribute(StrutsCXConstants.XSL_KEY, "acube.erreurXML.transform");
		request.setAttribute(StrutsCXConstants.XML_KEY, errors);

        return mapping.findForward("error");
	}
}
