package acube.gram.frontend.action;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import acube.framework.exception.FunctionalException;
import acube.framework.exception.TechnicalException;
import acube.framework.utility.GetRequest;
import acube.gram.backend.ExecutionContext;
import acube.gram.business.Delegate;
import acube.gram.constant.AttributeKeys;
import acube.gram.constant.ReportFormat;
import acube.gram.constant.ReportFormatCodec;
import acube.gram.exception.GramErrorCodes;
import acube.gram.exception.GramException;
import acube.gram.exception.GramExceptionFactory;
import acube.gram.util.CookieWrapper;
import acube.gram.util.SimpleCache;

/**
 * Regroupe les accs aux paramtres de la requte pour les actions de GRaM
 * @author MAE
 *
 */
public class ParameterAccessor {

	private static final String PARAM_REPORT = "REPORT";
	private static final String PARAM_FORMAT = "FORMAT";
	private static final String PARAM_ITEM_ID = "ID";
	private static final String PARAM_RESOURCE_ID = "ID";
	public static final String COOKIE_NAME = "acube.gram.ExecutionID";
	
	public static String getReport(HttpServletRequest request)
			throws FunctionalException, TechnicalException {
		
		return GetRequest.getNotEmptyString(request, PARAM_REPORT);
	}
	
	public static ReportFormat getReportFormat(HttpServletRequest request)
			throws FunctionalException, TechnicalException {
		
		return ReportFormatCodec.decode(GetRequest.getNotEmptyString(request, PARAM_FORMAT));
	}
	
	public static String getItemId(HttpServletRequest request)
			throws FunctionalException, TechnicalException {
		
		return GetRequest.getNotEmptyString(request, PARAM_ITEM_ID);
	}
	
	public static String getResourceId(HttpServletRequest request)
			throws FunctionalException, TechnicalException {
		
		return GetRequest.getNotEmptyString(request, PARAM_RESOURCE_ID);
	}
	
	/**
	 * met tous les paramtres de la requte dans un Map,  l'exception des paramtres
	 * utiliss par les actions. Les paramtres restant sont supposs paramtres du rapport.
	 * @param request
	 * @return
	 */
	public static Map<String, String> getParameterValues(HttpServletRequest request) {
		Map<String,String> parameters = new HashMap<String,String>();
		
		// on rcupre les paramtres du rapport
		Enumeration iParams = request.getParameterNames();
		while (iParams.hasMoreElements()) {

			String name = (String)iParams.nextElement();
			// s'il n'est pas pour le rapport, on l'ignore
			if (!name.startsWith("_"))
				continue;

			// on enlve le "_"
			parameters.put(name.substring(1),request.getParameter(name));
		}
		return parameters;
	}
	
	/**
	 * renvoie la liste la valeur d'un paramtre sous forme de liste (sparateur=virgule)
	 * @param request
	 * @param param
	 * @return
	 * @throws TechnicalException
	 * @throws FunctionalException
	 */
	public static List<String> getStringList(HttpServletRequest request,String param)
			throws TechnicalException, FunctionalException {
		
		String value = GetRequest.getString(request, param);
		ArrayList<String> result = new ArrayList<String>();
		
		if (!value.equals("")) {
			String values[] = value.split(",");
			for (int i=0; i<values.length; i++)
				result.add(values[i]);
		}
		
		return result;
	}
	
	public static Delegate getDelegate(HttpServletRequest request)
			throws GramException {
	
		Delegate delegate = (Delegate) request
			.getSession()
			.getServletContext()
			.getAttribute(AttributeKeys.GRAM_DELEGATE);

		if (delegate==null)
			throw GramExceptionFactory.createConfigurationError("GRaM delegate is missing");
		
		return delegate;
	}
	
	public static ExecutionContext getExecutionContext(HttpServletRequest request)
			throws FunctionalException, TechnicalException {
	
		String executionID = CookieWrapper.readCookie(request, COOKIE_NAME);
		if (executionID==null)
			throw new GramException(GramErrorCodes.INVALID_EXECUTIONID);
	
		ExecutionContext context = (ExecutionContext)
			SimpleCache.get(request.getSession(),
							 AttributeKeys.EXECUTION_CONTEXTS,
							 executionID);
		if (context==null)
			throw new GramException(GramErrorCodes.INVALID_EXECUTIONID);
		return context;
	}
	
	public static void setExecutionContext(HttpServletRequest request, HttpServletResponse response, ExecutionContext context)
			throws TechnicalException {
		
		String executionID = SimpleCache.add(request.getSession(),AttributeKeys.EXECUTION_CONTEXTS,context);
		// on met le cookie  la racine de l'application
		String path = request.getContextPath();
		CookieWrapper.setCookie(response, path, COOKIE_NAME, executionID);
	}
}
