package acube.gram2.business;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import acube.framework.exception.DAOException;
import acube.framework.exception.JDBCWrapperException;
import acube.framework.exception.TechnicalException;
import acube.gram.backend.Backend;
import acube.gram.backend.ExecutionContext;
import acube.gram.business.Delegate;
import acube.gram.exception.GramErrorCodes;
import acube.gram.exception.GramException;
import acube.gram2.vo.NodeVO;
import acube.gram2.vo.ReportVO;
import acube.gram2.vo.RoleVO;

public abstract class AbstractDelegateImpl implements Delegate {

	private Backend backend;
	
	public AbstractDelegateImpl(Backend backend) {

		this.backend = backend;
	}
	
	public abstract boolean isAdministrator(HttpServletRequest request)
		throws GramException;

	public abstract boolean isManager(HttpServletRequest request)
		throws GramException;
	
	public void assertAdministrator(HttpServletRequest request)
		throws GramException {
		
		if (!isAdministrator(request))
			throw new GramException(GramErrorCodes.AUTHORIZATION_ERROR);
	}

	public void assertManager(HttpServletRequest request)
		throws GramException {
			
		if (!isManager(request))
			throw new GramException(GramErrorCodes.AUTHORIZATION_ERROR);
	}

	
/*
	public List<FileMetadata> listFiles(HttpServletRequest request, FileType type)
			throws GramException {
		
		assertManager(request);
		return backend.listFiles();
	}
	
	public void createFile(HttpServletRequest request, FileType type, String chemin, String contentType, byte[] definition)
			throws GramException {

		assertAdministrator(request);
		backend.createFile(type, chemin, contentType, definition);
	}
	
	public FileVO readFile(HttpServletRequest request, FileType type, String racine)
			throws GramException {

		assertAdministrator(request);
		return backend.readFile(type, racine);
	}
	
	public void updateFile(HttpServletRequest request, FileType type, String chemin, String mimeType, byte[] definition)
			throws GramException {

		assertAdministrator(request);
		backend.updateFile(type, chemin, mimeType, definition);
	}
	
	public void deleteFile(HttpServletRequest request, FileType type, String chemin)
			throws GramException {

		assertAdministrator(request);
		backend.deleteFile(type, chemin);
	}
*/
	
	
	
	
	public ExecutionContext loadReport(HttpServletRequest request, String report)
			throws GramException {

		return backend.loadReport(request, report);
	}
	
	


	public abstract List<RoleVO> listRoles(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract ReportVO getReportById(HttpServletRequest request, String id)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract List<ReportVO> listReports(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void createReport(HttpServletRequest request, String path, String name, List<String> roles)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void updateReport(HttpServletRequest request, String path, String name, List<String> roles)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void deleteReport(HttpServletRequest request, String path)
		throws TechnicalException, JDBCWrapperException, DAOException;
	
	
	
	
	
	public abstract List<NodeVO> listNodes(HttpServletRequest request)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void createNode(HttpServletRequest request, Integer parent, String name, List<String> reports)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void updateNode(HttpServletRequest request, int id, String name, List<String> reports)
		throws TechnicalException, JDBCWrapperException, DAOException;

	public abstract void deleteNode(HttpServletRequest request, int id)
		throws TechnicalException, JDBCWrapperException, DAOException;
}
