package acube.gram2.integration;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import acube.framework.exception.JDBCWrapperException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.jdbc.JDBCWrapper;
import acube.gram2.vo.NodeVO;
import acube.gram2.vo.ReportVO;
import acube.gram2.vo.RoleVO;

public class DAOJdbc implements DAO {

	private String reportTable = "rapport";
	private String reportPath = "ra_p_chemin";
	private String reportName = "ra_l_nom";

	private String roleTable = "profil";
	private String roleId = "pr_id";
	private String roleName = "pr_l_nom";
	
	private String nodeTable = "noeud";
	private String nodeId = "nd_id";
	private String nodeParent = "parent_nd_id";
	private String nodeName = "nd_l_nom";
	
	private String reportRoleTable = "rapport_profil";
	private String nodeReportTable = "rapport_noeud";
	
	/*
	 * Report interface
	 */
	
	private String createSelectForReport() {
		
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		sb.append("report."+reportPath);
		sb.append(", ");
		sb.append("report."+reportName);
		sb.append(" FROM ");
		sb.append(reportTable);
		sb.append(" AS report ");
		return sb.toString();
	}
	
	protected ReportVO createReportVO(JDBCWrapper j)
			throws TechnicalException, JDBCWrapperException {

    	ReportVO vo = new ReportVO();
    	vo.setPath(j.getString(reportPath));
    	vo.setName(j.getString(reportName));
    	return vo;
    }
    
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#listReports()
	 */
	public List<ReportVO> listReports()
			throws JDBCWrapperException, TechnicalException {
		
		String sql = createSelectForReport();
		Vector params = new Vector();

		ArrayList<ReportVO> result = new ArrayList<ReportVO>();
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		while (jdbcWrapper.next())
			result.add(createReportVO(jdbcWrapper));
		jdbcWrapper.close();
		
		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#getReportById(java.lang.String)
	 */
	public ReportVO getReportById(String path)
			throws JDBCWrapperException, TechnicalException {
		
		String sql = createSelectForReport()+ " WHERE " + reportPath + "=?";
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		ReportVO result;
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		jdbcWrapper.next();
		result = createReportVO(jdbcWrapper);
		jdbcWrapper.close();
		
		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#addReport(java.lang.String, java.lang.String)
	 */
	public void addReport(String path, String name)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("INSERT INTO ");
		sb.append(reportTable);
		sb.append("(");
		sb.append(reportPath);
		sb.append(", ");
		sb.append(reportName);
		sb.append(") VALUES (?,?)");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		params.add(name);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#updateReport(java.lang.String, java.lang.String)
	 */
	public void updateReport(String path, String name)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("UPDATE ");
		sb.append(reportTable);
		sb.append(" SET ");
		sb.append(reportName);
		sb.append("=? WHERE ");
		sb.append(reportPath);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(name);
		params.add(path);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#deleteReport(java.lang.String)
	 */
	public void deleteReport(String path)
			throws JDBCWrapperException, TechnicalException {
			
		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(reportTable);
		sb.append(" WHERE ");
		sb.append(reportPath);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#listReportsForNode(int)
	 */
	public List<ReportVO> listReportsForNode(int id)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append(createSelectForReport());
		sb.append(" INNER JOIN ");
		sb.append(nodeReportTable);
		sb.append(" AS nodeReport ON ");
		sb.append("nodeReport."+reportPath);
		sb.append("=");
		sb.append("report."+reportPath);
		sb.append(" WHERE ");
		sb.append("nodeReport."+nodeId+"=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(id);

		ArrayList<ReportVO> result = new ArrayList<ReportVO>();
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		while (jdbcWrapper.next())
			result.add(createReportVO(jdbcWrapper));
		jdbcWrapper.close();
		
		return result;
	}

	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#addReportToNode(int, java.lang.String)
	 */
	public void addReportToNode(int id, String report)
			throws JDBCWrapperException, TechnicalException {

		StringBuffer sb = new StringBuffer();
		sb.append("INSERT INTO ");
		sb.append(nodeReportTable);
		sb.append("(");
		sb.append(nodeId);
		sb.append(",");
		sb.append(reportPath);
		sb.append(") VALUES (?,?)");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(id);
		params.add(report);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#removeReportFromNode(int, java.lang.String)
	 */
	public void removeReportFromNode(int id, String report)
			throws JDBCWrapperException, TechnicalException {

		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(nodeReportTable);
		sb.append(" WHERE ");
		sb.append(nodeId);
		sb.append("=? AND ");
		sb.append(reportPath);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(id);
		params.add(report);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#removeAllReportsFromNode(int)
	 */
	public void removeAllReportsFromNode(int id)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(nodeReportTable);
		sb.append(" WHERE ");
		sb.append(nodeId);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(id);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	public void removeReportFromAllNodes(String path)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(nodeReportTable);
		sb.append(" WHERE ");
		sb.append(reportPath);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	
	
	
	/*
	 * Node interface
	 */
	
	private String createSelectForNode() {
		
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		sb.append("node."+nodeId);
		sb.append(", ");
		sb.append("node."+nodeParent);
		sb.append(", ");
		sb.append("node."+nodeName);
		sb.append(" FROM ");
		sb.append(nodeTable);
		sb.append(" AS node ");
		return sb.toString();
	}
	
	protected NodeVO createNodeVO(JDBCWrapper j)
			throws TechnicalException, JDBCWrapperException {

    	NodeVO vo = new NodeVO();
    	vo.setId(j.getInt(nodeId));
    	String parentIdAsString = j.getString(nodeParent);
    	if (parentIdAsString != null)
    		vo.setParent(new Integer(parentIdAsString));
    	vo.setName(j.getString(nodeName));
    	return vo;
    }
    
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#listNodes()
	 */
	public List<NodeVO> listNodes()
			throws JDBCWrapperException, TechnicalException {
		
		String sql = createSelectForNode();
		Vector params = new Vector();

		ArrayList<NodeVO> result = new ArrayList<NodeVO>();
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		while (jdbcWrapper.next())
			result.add(createNodeVO(jdbcWrapper));
		jdbcWrapper.close();
		
		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#getNodeById(int)
	 */
	public NodeVO getNodeById(int id)
			throws JDBCWrapperException, TechnicalException {
		
		String sql = createSelectForNode()+ " WHERE " + nodeId + "=?";
		Vector<Object> params = new Vector<Object>();
		params.add(id);
		NodeVO result;
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		jdbcWrapper.next();
		result = createNodeVO(jdbcWrapper);
		jdbcWrapper.close();
		
		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#addNode(java.lang.Integer, java.lang.String)
	 */
	public int addNode(Integer parent, String name)
			throws JDBCWrapperException, TechnicalException {
		
		// insert the new row
		StringBuffer sb = new StringBuffer();
		sb.append("INSERT INTO ");
		sb.append(nodeTable);
		sb.append("(");
		sb.append(nodeParent);
		sb.append(", ");
		sb.append(nodeName);
		sb.append(") VALUES (?,?)");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(parent);
		params.add(name);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
		
		// get the id of last row with the same name
		sb = new StringBuffer();
		sb.append("SELECT TOP 1 ");
		sb.append(nodeId);
		sb.append(" FROM ");
		sb.append(nodeTable);
		sb.append(" WHERE ");
		sb.append(nodeName);
		sb.append("=? ORDER BY ");
		sb.append(nodeId);
		sb.append(" DESC");

		sql = sb.toString();
		params = new Vector<Object>();
		params.add(name);
		
		jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		jdbcWrapper.next();
		int result = jdbcWrapper.getInt(nodeId);
		jdbcWrapper.close();

		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#updateNode(int, java.lang.String)
	 */
	public void updateNode(int id, String name)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("UPDATE ");
		sb.append(nodeTable);
		sb.append(" SET ");
		sb.append(nodeName);
		sb.append("=? WHERE ");
		sb.append(nodeId);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(name);
		params.add(id);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#deleteNode(int)
	 */
	public void deleteNode(int id)
			throws JDBCWrapperException, TechnicalException {
			
		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(nodeTable);
		sb.append(" WHERE ");
		sb.append(nodeId);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(id);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	/*
	 * Role interface
	 */
	
	private String createSelectForRole() {
		
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		sb.append("role."+roleId);
		sb.append(", ");
		sb.append("role."+roleName);
		sb.append(" FROM ");
		sb.append(roleTable);
		sb.append(" AS role ");
		return sb.toString();
	}
	
	protected RoleVO createRoleVO(JDBCWrapper j)
			throws TechnicalException, JDBCWrapperException {

    	RoleVO vo = new RoleVO();
    	vo.setId(j.getString(roleId));
    	vo.setName(j.getString(roleName));
    	return vo;
    }
    
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#listRoles()
	 */
	public List<RoleVO> listRoles()
			throws JDBCWrapperException, TechnicalException {
		
		String sql = createSelectForRole();
		Vector params = new Vector();

		ArrayList<RoleVO> result = new ArrayList<RoleVO>();
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		while (jdbcWrapper.next())
			result.add(createRoleVO(jdbcWrapper));
		jdbcWrapper.close();
		
		return result;
	}

	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#listRolesForReport(java.lang.String)
	 */
	public List<RoleVO> listRolesForReport(String path)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append(createSelectForRole());
		sb.append(" INNER JOIN ");
		sb.append(reportRoleTable);
		sb.append(" AS reportRole ON ");
		sb.append("reportRole."+roleId);
		sb.append("=");
		sb.append("role."+roleId);
		sb.append(" WHERE ");
		sb.append("reportRole."+reportPath+"=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);

		ArrayList<RoleVO> result = new ArrayList<RoleVO>();
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_QUERY, params);
		while (jdbcWrapper.next())
			result.add(createRoleVO(jdbcWrapper));
		jdbcWrapper.close();
		
		return result;
	}

	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#addRoleToReport(java.lang.String, java.lang.String)
	 */
	public void addRoleToReport(String path, String role)
			throws JDBCWrapperException, TechnicalException {

		StringBuffer sb = new StringBuffer();
		sb.append("INSERT INTO ");
		sb.append(reportRoleTable);
		sb.append("(");
		sb.append(reportPath);
		sb.append(",");
		sb.append(roleId);
		sb.append(") VALUES (?,?)");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		params.add(role);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#removeRoleFromReport(java.lang.String, java.lang.String)
	 */
	public void removeRoleFromReport(String path, String role)
			throws JDBCWrapperException, TechnicalException {

		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(reportRoleTable);
		sb.append(" WHERE ");
		sb.append(reportPath);
		sb.append("=? AND ");
		sb.append(roleId);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		params.add(role);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
	
	/* (non-Javadoc)
	 * @see acube.gram.integration.DAO#removeAllRolesFromReport(java.lang.String)
	 */
	public void removeAllRolesFromReport(String path)
			throws JDBCWrapperException, TechnicalException {
		
		StringBuffer sb = new StringBuffer();
		sb.append("DELETE ");
		sb.append(reportRoleTable);
		sb.append(" WHERE ");
		sb.append(reportPath);
		sb.append("=?");
		
		String sql = sb.toString();
		Vector<Object> params = new Vector<Object>();
		params.add(path);
		
		JDBCWrapper jdbcWrapper = new JDBCWrapper();
		jdbcWrapper.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE, params);
	}
}
